/**
 *  PSPad cue sheet to audacity labels conversion script
 *  (c) 2008 by rsbrux 
 */

var module_name = "CueSheetConvertJS";
var module_ver = "0.51";
var fs = new ActiveXObject("Scripting.FileSystemObject");
var rTitle = new RegExp("TITLE");
var rTrack = new RegExp("TRACK");
var rIndex = new RegExp("INDEX 01");
var rPregap = new RegExp("PREGAP");
var nPregap;
var nSecondsOut;
var nTrack;
var nSecondsOut;

function ParseLine(sIn, edOut) {
  var sTxt;
  var x = sIn.search(rTrack);

  if (x >= 0)
  {
    x += 6;
    nTrack = parseInt(sIn.substr(x,2), 10);
  }
  else
  {
    x = sIn.search(rTitle);
    if (x >= 0)
    {
      x += 7;
      sTxt = sIn.substr(x);
      sTitle = sTxt.replace(/"/, "");
    }
    else
    {
      x = sIn.search(rIndex);
      if (x >= 0)
      {
        x += 9;
        nSecondsOut = nPregap + ParseIndex(sIn, x);
        edOut.appendText(nSecondsOut.toFixed(6));
        edOut.appendText(" ");
        edOut.appendText(sTitle);
      }
      else
      {
        if (nTrack == 1) //PREGAP respected only before 1st track
        {
          x = sIn.search(rPregap);
          if (x >= 0)
          {
            x += 7;
            nPregap = ParseIndex(sIn, x);
          }
        }
      }
    }
  }
}

function ParseIndex(strIn, nPos) {
  var nMinutes;
  var nSeconds;
  var nFrames;

  nMinutes = parseInt(strIn.substr(nPos,2), 10);
  nSeconds = parseInt(strIn.substr(nPos+3,2), 10);
  nFrames = parseInt(strIn.substr(nPos+6,2), 10);
  return (nMinutes*60 + nSeconds + nFrames/74);
}

function ConvertCuesheetToLabels() {
  var edLog = newEditor();
  edLog.assignActiveEditor();

  var edLabels = newEditor();
  edLabels.NewFile();

  var i;
  var l = s.split("\n");
  var s = edLog.Text();
  var sLine;
  nPregap = 0;

  for (i in l)
  {
    sLine = l[i];
    ParseLine(sLine, edLabels);
  }
}

function ConvertCuesheetFileToLabels(source, destination) {

  var fr = fs.GetFile(source).OpenAsTextStream(1, 0);
  var i = 0;
  var sLine;
  var txt;

  var edLabels = newEditor();
  edLabels.NewFile(destination);

  nPregap = 0;

  while (!fr.AtEndOfStream) {
    i++;
    txt = fr.ReadLine();
    sLine = txt + "\n";
    ParseLine(sLine, edLabels);
  }
  fr.Close( );
}

function ConvertCuesheetProjectToLabels() {
  var i;
  var filename = "";
  var fr;
  var rC = new RegExp(".cue", i);

  for (i = 0; i < (projectFilesCount()-1); i++) {
    fr = projectFiles(i);
    if (fs.FileExists(fr) && rC.test(fr)) {
      var r = fr.search(rC);
      filename = fr.replace(rC, ".txt");
      ConvertCuesheetFileToLabels(fr, filename);
    }
  }
}

function Init(){
  addMenuItem("Convert Cuesheet to Labels", "Cuesheet Converter", "ConvertCuesheetToLabels");
  addMenuItem("Convert Cuesheet Project to Labels", "Cuesheet Converter", "ConvertCuesheetProjectToLabels");
}
